import datetime

try:
    import memcache
    memcache_installed = True
except ImportError:
    memcache_installed = False

def get_memcached_stats(server):
    if not memcache_installed:
        return {}
    host = memcache._Host(server)
    host.connect()
    host.send_cmd("stats")
    
    stats = {}
    
    while True:
        try:
            stat, key, value = host.readline().split(None, 2)
        except ValueError:
            break
        try:
            # Convert to native type, if possible
            value = int(value)
            if key == "uptime":
                value = datetime.timedelta(seconds=value)
            elif key == "time":
                value = datetime.datetime.fromtimestamp(value)
        except ValueError:
            pass
        stats[key] = value

    host.close_socket()
    
    try:
        stats['hit_rate'] = 100 * stats['get_hits'] / stats['cmd_get']
    except ZeroDivisionError:
        stats['hit_rate'] = stats['get_hits']
    
    return stats


from setuptools import setup, find_packages
 
version = '0.1.2'
 
LONG_DESCRIPTION = """
django-memcached
=================

This is a very simple reusable app which does one thing: shows you statistics
about your currently running memcached instances.  There are no database models
and not much configuration.

Installing django-memcached
----------------------------

1. Either download the tarball and run ``python setup.py install``, or simply
   use easy install or pip like so ``easy_install django-memcached``.


2. Add django_memcached to your ``INSTALLED_APPS`` setting::

       INSTALLED_APPS = (
           # ...
           'django_memcached',
       )


3. Add the urls to your url configuration::

       urlpatterns = patterns('',
           # ...
           (r'^cache/', include('django_memcached.urls')),
       )


4. If you want to restrict these urls to only staff members, just add this
   setting to your settings file::
   
       DJANGO_MEMCACHED_REQUIRE_STAFF = True


That's it.  Hopefully this app gives you a little insight into how your
memcached servers are being utilized.
"""
 
setup(
    name='django-memcached',
    version=version,
    description="django-memcached is a very simple reusable app which does one simple thing: shows you statistics about your currently running memcached instances",
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
    ],
    keywords='memcached,django',
    author='Eric Florenzano',
    author_email='floguy@gmail.com',
    url='http://github.com/ericflo/django-memcached/tree/master',
    license='BSD',
    packages=find_packages(),
    zip_safe=False,
    install_requires=['setuptools'],
    include_package_data=True,
    setup_requires=['setuptools_git'],
)


from django.http import Http404
from django.shortcuts import render_to_response
from django.conf import settings
from django.template import RequestContext

from django_memcached.util import get_memcached_stats
from django.contrib.auth.decorators import user_passes_test

SERVERS = settings.CACHE_BACKEND.replace('memcached://', '').replace('/','').split(';')

def server_list(request):
    statuses = zip(range(len(SERVERS)), SERVERS, map(get_memcached_stats, SERVERS))
    context = {
        'statuses': statuses,
    }
    return render_to_response(
        'memcached/server_list.html',
        context,
        context_instance=RequestContext(request)
    )

def server_status(request, index):
    try:
        index = int(index)
    except ValueError:
        raise Http404
    if 'memcached' not in settings.CACHE_BACKEND:
        raise Http404
    if not SERVERS:
        raise Http404
    try:
        server = SERVERS[index]
    except IndexError:
        raise Http404
    stats = get_memcached_stats(server)
    if not stats:
        raise Http404
    context = {
        'server': server,
        'stats': stats.items(),
    }
    return render_to_response(
        'memcached/server_status.html',
        context,
        context_instance=RequestContext(request)
    )

if getattr(settings, 'DJANGO_MEMCACHED_REQUIRE_STAFF', False):
    server_list = user_passes_test(lambda u: u.is_staff)(server_list)
    server_status = user_passes_test(lambda u: u.is_staff)(server_status)